@@ -27,7 +27,6 @@
  *
  */
 
-#include <ldap.h>
 #include <getopt.h>
 #include <signal.h>
 #include <ctype.h>
@@ -62,7 +61,7 @@ int main (int argc, char ** argv) {
   config->log_mode = Y_LOG_MODE_NONE;
   config->log_level = Y_LOG_LEVEL_NONE;
   config->log_file = NULL;
-  config->use_scope = 0;
+  config->use_scope = 1;
   config->conn = NULL;
   config->instance = o_malloc(sizeof(struct _u_instance));
   config->allow_origin = NULL;
@@ -73,6 +72,7 @@ int main (int argc, char ** argv) {
   config->refresh_token_expiration = GLEWLWYD_REFRESH_TOKEN_EXP_DEFAULT;
   config->access_token_expiration = GLEWLWYD_ACCESS_TOKEN_EXP_DEFAULT;
   config->code_expiration = GLEWLWYD_CODE_EXPIRATION_DEFAULT;
+  config->auth_code_match_ip_address = 1;
   config->jwt_decode_key = NULL;
   config->jwt = NULL;
   config->session_key = o_strdup(GLEWLWYD_SESSION_KEY_DEFAULT);
@@ -114,7 +114,7 @@ int main (int argc, char ** argv) {
     fprintf(stderr, "init - Error initializing end signal\n");
     return 1;
   }
-	
+  
   // First we parse command line arguments
   if (!build_config_from_args(argc, argv, config)) {
     fprintf(stderr, "Error reading command-line parameters\n");
@@ -433,11 +433,13 @@ int build_config_from_args(int argc, char ** argv, struct config_elements * conf
             one_log_mode = strtok(tmp, ",");
             while (one_log_mode != NULL) {
               if (0 == strncmp("console", one_log_mode, strlen("console"))) {
-                config->log_mode |= Y_LOG_MODE_CONSOLE;
+                config->log_mode += Y_LOG_MODE_CONSOLE;
               } else if (0 == strncmp("syslog", one_log_mode, strlen("syslog"))) {
-                config->log_mode |= Y_LOG_MODE_SYSLOG;
+                config->log_mode += Y_LOG_MODE_SYSLOG;
+              } else if (0 == strncmp("journald", one_log_mode, strlen("journald"))) {
+                config->log_mode += Y_LOG_MODE_JOURNALD;
               } else if (0 == strncmp("file", one_log_mode, strlen("file"))) {
-                config->log_mode |= Y_LOG_MODE_FILE;
+                config->log_mode += Y_LOG_MODE_FILE;
               }
               one_log_mode = strtok(NULL, ",");
             }
@@ -478,8 +480,11 @@ int build_config_from_args(int argc, char ** argv, struct config_elements * conf
           }
           break;
         case 'h':
+          print_help(stdout);
+          exit_server(&config, GLEWLWYD_STOP);
+          break;
         case 'v':
-					print_help(stdout);
+          fprintf(stdout, "%s\n", _GLEWLWYD_VERSION_);
           exit_server(&config, GLEWLWYD_STOP);
           break;
       }
@@ -507,7 +512,7 @@ void print_help(FILE * output) {
   fprintf(output, "\n");
   fprintf(output, "Version %s\n", _GLEWLWYD_VERSION_);
   fprintf(output, "\n");
-  fprintf(output, "Copyright 2016-2017 Nicolas Mora <mail@babelouest.org>\n");
+  fprintf(output, "Copyright 2016-2018 Nicolas Mora <mail@babelouest.org>\n");
   fprintf(output, "\n");
   fprintf(output, "This program is free software; you can redistribute it and/or\n");
   fprintf(output, "modify it under the terms of the GNU GENERAL PUBLIC LICENSE\n");
@@ -533,8 +538,9 @@ void print_help(FILE * output) {
   fprintf(output, "\tdefault: ERROR\n");
   fprintf(output, "-f --log-file=PATH\n");
   fprintf(output, "\tPath for log file if log mode file is specified\n");
-  fprintf(output, "-h --help\n");
   fprintf(output, "-v --version\n");
+  fprintf(output, "\tPrint Glewlwyd's current version\n\n");
+  fprintf(output, "-h --help\n");
   fprintf(output, "\tPrint this message\n\n");
 }
 
@@ -592,10 +598,10 @@ int build_config_from_file(struct config_elements * config) {
              * db_type = NULL, * db_sqlite_path = NULL, * db_mariadb_host = NULL, * db_mariadb_user = NULL, * db_mariadb_password = NULL,
              * db_mariadb_dbname = NULL, * cur_allow_origin = NULL, * cur_static_files_path = NULL, * cur_static_files_prefix = NULL,
              * cur_session_key = NULL, * cur_admin_scope = NULL, * cur_profile_scope = NULL, * cur_additional_property_name = NULL, 
-             * cur_auth_ldap_uri = NULL, * cur_auth_ldap_bind_dn = NULL, * cur_auth_ldap_bind_passwd = NULL,
+             * cur_auth_ldap_uri = NULL, * cur_auth_ldap_bind_dn = NULL, * cur_auth_ldap_bind_passwd = NULL, * cur_auth_ldap_search_scope = NULL,
              * cur_auth_ldap_base_search_user = NULL, * cur_auth_ldap_filter_user_read = NULL,
              * cur_auth_ldap_login_property_user_read = NULL, * cur_auth_ldap_name_property_user_read = NULL,
-             * cur_auth_ldap_email_property_user_read = NULL, * cur_auth_ldap_scope_property_user_read = NULL,
+             * cur_auth_ldap_email_property_user_read = NULL, * cur_auth_ldap_scope_property_user_read = NULL, * cur_auth_ldap_scope_property_user_match = NULL,
              * cur_auth_ldap_additional_property_value_read = NULL, * cur_auth_ldap_additional_property_value_write = NULL,
              * cur_auth_ldap_rdn_property_user_write = NULL, * cur_auth_ldap_login_property_user_write = NULL,
              * cur_auth_ldap_name_property_user_write = NULL, * cur_auth_ldap_email_property_user_write = NULL,
@@ -606,7 +612,7 @@ int build_config_from_file(struct config_elements * config) {
              * cur_auth_ldap_scope_property_client_read = NULL, * cur_auth_ldap_description_property_client_read = NULL,
              * cur_auth_ldap_redirect_uri_property_client_read = NULL, * cur_auth_ldap_confidential_property_client_read = NULL,
              * cur_auth_ldap_client_id_property_client_write = NULL, * cur_auth_ldap_rdn_property_client_write = NULL,
-             * cur_auth_ldap_name_property_client_write = NULL, * cur_auth_ldap_scope_property_client_write = NULL,
+             * cur_auth_ldap_name_property_client_write = NULL, * cur_auth_ldap_scope_property_client_write = NULL, * cur_auth_ldap_scope_property_client_match = NULL,
              * cur_auth_ldap_description_property_client_write = NULL, * cur_auth_ldap_redirect_uri_property_client_write = NULL,
              * cur_auth_ldap_confidential_property_client_write = NULL, * cur_auth_ldap_password_property_client_write = NULL,
              * cur_auth_ldap_password_algorithm_client_write = NULL, * cur_auth_ldap_object_class_client_write = NULL,
@@ -617,7 +623,14 @@ int build_config_from_file(struct config_elements * config) {
              * cur_reset_password_email_template_path = NULL, * cur_reset_password_page_url_prefix = NULL,
              * cur_auth_http_auth_url = NULL;
   int db_mariadb_port = 0, cur_key_size = 512;
-  int cur_http_auth = 0, cur_database_auth = 0, cur_ldap_auth = 0, cur_use_scope = 0, cur_use_rsa = 0, cur_use_ecdsa = 0, cur_use_sha = 0, cur_use_secure_connection = 0, cur_auth_ldap_user_write = 0, cur_auth_ldap_client_write = 0, cur_auth_http_check_server_certificate = 1, i;
+  int cur_http_auth = 0, cur_database_auth = 0, cur_ldap_auth = 0, cur_use_scope = 0, cur_use_rsa = 0, cur_use_ecdsa = 0, cur_use_sha = 0,
+      cur_use_secure_connection = 0, cur_auth_ldap_user_write = 0, cur_auth_ldap_client_write = 0, cur_auth_http_check_server_certificate = 1, i;
+  ber_int_t cur_auth_ldap_page_size = 0;
+  
+  // JWT autocheck
+  time_t now;
+  char * session_token;
+  jwt_t * jwt_check = NULL;
   
   config_init(&cfg);
   
@@ -652,11 +665,13 @@ int build_config_from_file(struct config_elements * config) {
       one_log_mode = strtok((char *)cur_log_mode, ",");
       while (one_log_mode != NULL) {
         if (0 == strncmp("console", one_log_mode, strlen("console"))) {
-          config->log_mode |= Y_LOG_MODE_CONSOLE;
+          config->log_mode += Y_LOG_MODE_CONSOLE;
         } else if (0 == strncmp("syslog", one_log_mode, strlen("syslog"))) {
-          config->log_mode |= Y_LOG_MODE_SYSLOG;
+          config->log_mode += Y_LOG_MODE_SYSLOG;
+        } else if (0 == strncmp("journald", one_log_mode, strlen("journald"))) {
+          config->log_mode += Y_LOG_MODE_JOURNALD;
         } else if (0 == strncmp("file", one_log_mode, strlen("file"))) {
-          config->log_mode |= Y_LOG_MODE_FILE;
+          config->log_mode += Y_LOG_MODE_FILE;
           // Get log file path
           if (config->log_file == NULL) {
             if (config_lookup_string(&cfg, "log_file", &cur_log_file)) {
@@ -668,6 +683,10 @@ int build_config_from_file(struct config_elements * config) {
               }
             }
           }
+        } else {
+          fprintf(stderr, "Error, logging mode '%s' unknown\n", one_log_mode);
+          config_destroy(&cfg);
+          return 0;
         }
         one_log_mode = strtok(NULL, ",");
       }
@@ -712,6 +731,7 @@ int build_config_from_file(struct config_elements * config) {
   config_lookup_int(&cfg, "access_token_expiration", (int *)&config->access_token_expiration);
   config_lookup_int(&cfg, "session_expiration", (int *)&config->session_expiration);
   config_lookup_int(&cfg, "code_expiration", (int *)&config->code_expiration);
+  config_lookup_bool(&cfg, "auth_code_match_ip_address", (int *)&config->auth_code_match_ip_address);
   
   config_lookup_string(&cfg, "session_key", &cur_session_key);
   if (cur_session_key != NULL) {
@@ -931,13 +951,13 @@ int build_config_from_file(struct config_elements * config) {
     if (config_setting_lookup_string(database, "type", &db_type) == CONFIG_TRUE) {
       if (0) {
         // I know, this is for the code below to work
-#ifdef _HOEL_MARIADB
+#ifdef _HOEL_SQLITE
         } else if (0 == strncmp(db_type, "sqlite3", strlen("sqlite3"))) {
         if (config_setting_lookup_string(database, "path", &db_sqlite_path) == CONFIG_TRUE) {
           config->conn = h_connect_sqlite(db_sqlite_path);
           if (config->conn == NULL) {
-            config_destroy(&cfg);
             fprintf(stderr, "Error opening sqlite database %s\n", db_sqlite_path);
+            config_destroy(&cfg);
             return 0;
           } else {
             if (h_exec_query_sqlite(config->conn, "PRAGMA foreign_keys = ON;") != H_OK) {
@@ -952,7 +972,7 @@ int build_config_from_file(struct config_elements * config) {
           return 0;
         }
 #endif
-#ifdef _HOEL_SQLITE
+#ifdef _HOEL_MARIADB
       } else if (0 == strncmp(db_type, "mariadb", strlen("mariadb"))) {
         config_setting_lookup_string(database, "host", &db_mariadb_host);
         config_setting_lookup_string(database, "user", &db_mariadb_user);
@@ -966,7 +986,7 @@ int build_config_from_file(struct config_elements * config) {
           return 0;
         }
 #endif
-        } else {
+      } else {
         config_destroy(&cfg);
         fprintf(stderr, "Error, database type unknown\n");
         return 0;
@@ -1026,6 +1046,8 @@ int build_config_from_file(struct config_elements * config) {
       config_setting_lookup_string(auth, "uri", &cur_auth_ldap_uri);
       config_setting_lookup_string(auth, "bind_dn", &cur_auth_ldap_bind_dn);
       config_setting_lookup_string(auth, "bind_passwd", &cur_auth_ldap_bind_passwd);
+      config_setting_lookup_string(auth, "search_scope", &cur_auth_ldap_search_scope);
+      config_setting_lookup_int(auth, "page_size", &cur_auth_ldap_page_size);
 
       config_setting_lookup_string(auth, "base_search_user", &cur_auth_ldap_base_search_user);
       config_setting_lookup_string(auth, "filter_user_read", &cur_auth_ldap_filter_user_read);
@@ -1035,6 +1057,7 @@ int build_config_from_file(struct config_elements * config) {
       config_setting_lookup_string(auth, "additional_property_value_read", &cur_auth_ldap_additional_property_value_read);
       if (config->use_scope) {
         config_setting_lookup_string(auth, "scope_property_user_read", &cur_auth_ldap_scope_property_user_read);
+        config_setting_lookup_string(auth, "scope_property_user_match", &cur_auth_ldap_scope_property_user_match);
       }
       
       config_setting_lookup_bool(auth, "ldap_user_write", &cur_auth_ldap_user_write);
@@ -1059,6 +1082,7 @@ int build_config_from_file(struct config_elements * config) {
       config_setting_lookup_string(auth, "confidential_property_client_read", &cur_auth_ldap_confidential_property_client_read);
       if (config->use_scope) {
         config_setting_lookup_string(auth, "scope_property_client_read", &cur_auth_ldap_scope_property_client_read);
+        config_setting_lookup_string(auth, "scope_property_client_match", &cur_auth_ldap_scope_property_client_match);
       }
       
       config_setting_lookup_bool(auth, "ldap_client_write", &cur_auth_ldap_client_write);
@@ -1075,48 +1099,48 @@ int build_config_from_file(struct config_elements * config) {
       config_setting_lookup_string(auth, "password_algorithm_client_write", &cur_auth_ldap_password_algorithm_client_write);
       config_setting_lookup_string(auth, "object_class_client_write", &cur_auth_ldap_object_class_client_write);
       
-      if (cur_auth_ldap_uri != NULL && 
-          cur_auth_ldap_bind_dn != NULL && 
-          cur_auth_ldap_bind_passwd != NULL && 
+      if (cur_auth_ldap_uri != NULL &&
+          cur_auth_ldap_bind_dn != NULL &&
+          cur_auth_ldap_bind_passwd != NULL &&
           
-          cur_auth_ldap_base_search_user != NULL && 
-          cur_auth_ldap_filter_user_read != NULL && 
-          cur_auth_ldap_login_property_user_read != NULL && 
-          cur_auth_ldap_name_property_user_read != NULL && 
-          cur_auth_ldap_email_property_user_read != NULL && 
-          cur_auth_ldap_additional_property_value_read != NULL && 
-          (cur_auth_ldap_scope_property_user_read != NULL || !config->use_scope) && 
+          cur_auth_ldap_base_search_user != NULL &&
+          cur_auth_ldap_filter_user_read != NULL &&
+          cur_auth_ldap_login_property_user_read != NULL &&
+          cur_auth_ldap_name_property_user_read != NULL &&
+          cur_auth_ldap_email_property_user_read != NULL &&
+          cur_auth_ldap_additional_property_value_read != NULL &&
+          (cur_auth_ldap_scope_property_user_read != NULL || !config->use_scope) &&
           
-          (!cur_auth_ldap_user_write || 
-          (cur_auth_ldap_rdn_property_user_write != NULL && 
-          cur_auth_ldap_login_property_user_write != NULL && 
-          cur_auth_ldap_name_property_user_write != NULL && 
-          cur_auth_ldap_email_property_user_write != NULL && 
-          cur_auth_ldap_additional_property_value_write != NULL && 
-          (cur_auth_ldap_scope_property_user_write != NULL || !config->use_scope) && 
-          cur_auth_ldap_password_property_user_write != NULL && 
-          cur_auth_ldap_password_algorithm_user_write != NULL && 
-          cur_auth_ldap_object_class_user_write != NULL)) && 
+          (!cur_auth_ldap_user_write ||
+          (cur_auth_ldap_rdn_property_user_write != NULL &&
+          cur_auth_ldap_login_property_user_write != NULL &&
+          cur_auth_ldap_name_property_user_write != NULL &&
+          cur_auth_ldap_email_property_user_write != NULL &&
+          cur_auth_ldap_additional_property_value_write != NULL &&
+          (cur_auth_ldap_scope_property_user_write != NULL || !config->use_scope) &&
+          cur_auth_ldap_password_property_user_write != NULL &&
+          cur_auth_ldap_password_algorithm_user_write != NULL &&
+          cur_auth_ldap_object_class_user_write != NULL)) &&
           
-          cur_auth_ldap_base_search_client != NULL && 
-          cur_auth_ldap_filter_client_read != NULL && 
-          cur_auth_ldap_client_id_property_client_read != NULL && 
-          cur_auth_ldap_name_property_client_read != NULL && 
-          cur_auth_ldap_description_property_client_read != NULL && 
-          cur_auth_ldap_redirect_uri_property_client_read != NULL && 
-          cur_auth_ldap_confidential_property_client_read != NULL && 
-          (cur_auth_ldap_scope_property_client_read != NULL || !config->use_scope) && 
+          cur_auth_ldap_base_search_client != NULL &&
+          cur_auth_ldap_filter_client_read != NULL &&
+          cur_auth_ldap_client_id_property_client_read != NULL &&
+          cur_auth_ldap_name_property_client_read != NULL &&
+          cur_auth_ldap_description_property_client_read != NULL &&
+          cur_auth_ldap_redirect_uri_property_client_read != NULL &&
+          cur_auth_ldap_confidential_property_client_read != NULL &&
+          (cur_auth_ldap_scope_property_client_read != NULL || !config->use_scope) &&
           
-          (!cur_auth_ldap_client_write || 
-          (cur_auth_ldap_rdn_property_client_write != NULL && 
-          cur_auth_ldap_client_id_property_client_write != NULL && 
-          cur_auth_ldap_name_property_client_write != NULL && 
-          cur_auth_ldap_description_property_client_write != NULL && 
-          cur_auth_ldap_redirect_uri_property_client_write != NULL && 
-          cur_auth_ldap_confidential_property_client_write != NULL && 
-          (cur_auth_ldap_scope_property_client_write != NULL || !config->use_scope) && 
-          cur_auth_ldap_password_property_client_write != NULL && 
-          cur_auth_ldap_password_algorithm_client_write != NULL && 
+          (!cur_auth_ldap_client_write ||
+          (cur_auth_ldap_rdn_property_client_write != NULL &&
+          cur_auth_ldap_client_id_property_client_write != NULL &&
+          cur_auth_ldap_name_property_client_write != NULL &&
+          cur_auth_ldap_description_property_client_write != NULL &&
+          cur_auth_ldap_redirect_uri_property_client_write != NULL &&
+          cur_auth_ldap_confidential_property_client_write != NULL &&
+          (cur_auth_ldap_scope_property_client_write != NULL || !config->use_scope) &&
+          cur_auth_ldap_password_property_client_write != NULL &&
+          cur_auth_ldap_password_algorithm_client_write != NULL &&
           cur_auth_ldap_object_class_client_write != NULL))) {
         config->auth_ldap = o_malloc(sizeof(struct _auth_ldap));
         if (config->auth_ldap == NULL) {
@@ -1125,6 +1149,8 @@ int build_config_from_file(struct config_elements * config) {
           return 0;
         } else {
           memset(config->auth_ldap, 0, sizeof(struct _auth_ldap));
+          config->auth_ldap->search_scope = LDAP_SCOPE_ONELEVEL;
+          config->auth_ldap->page_size = GLEWLWYD_DEFAULT_LDAP_PAGE_SIZE;
           
           config->auth_ldap->uri = o_strdup(cur_auth_ldap_uri);
           if (config->auth_ldap->uri == NULL) {
@@ -1144,6 +1170,23 @@ int build_config_from_file(struct config_elements * config) {
             fprintf(stderr, "Error allocating resources for config->auth_ldap->bind_passwd\n");
             return 0;
           }
+          if (cur_auth_ldap_search_scope != NULL) {
+            if (0 == o_strcmp("onelevel", cur_auth_ldap_search_scope)) {
+              config->auth_ldap->search_scope = LDAP_SCOPE_ONELEVEL;
+            } else if (0 == o_strcmp("subtree", cur_auth_ldap_search_scope)) {
+              config->auth_ldap->search_scope = LDAP_SCOPE_SUBTREE;
+            } else if (0 == o_strcmp("children", cur_auth_ldap_search_scope)) {
+              config->auth_ldap->search_scope = LDAP_SCOPE_CHILDREN;
+            } else {
+              config_destroy(&cfg);
+              fprintf(stderr, "Error search_scope error, values available are 'onelevel', 'subtree' or 'children'\n");
+              return 0;
+            }
+          }
+          
+          if (cur_auth_ldap_page_size > 0) {
+            config->auth_ldap->page_size = cur_auth_ldap_page_size;
+          }
           
           config->auth_ldap->base_search_user = o_strdup(cur_auth_ldap_base_search_user);
           if (config->auth_ldap->base_search_user == NULL) {
@@ -1170,6 +1213,21 @@ int build_config_from_file(struct config_elements * config) {
               fprintf(stderr, "Error allocating resources for config->auth_ldap->scope_property_user_read\n");
               return 0;
             }
+            if (cur_auth_ldap_scope_property_user_match != NULL) {
+              if (0 == o_strcasecmp("equals", cur_auth_ldap_scope_property_user_match)) {
+                config->auth_ldap->scope_property_user_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_EQUALS;
+              } else if (0 == o_strcasecmp("contains", cur_auth_ldap_scope_property_user_match)) {
+                config->auth_ldap->scope_property_user_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_CONTAINS;
+              } else if (0 == o_strcasecmp("startswith", cur_auth_ldap_scope_property_user_match)) {
+                config->auth_ldap->scope_property_user_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_STARTSWITH;
+              } else if (0 == o_strcasecmp("endswith", cur_auth_ldap_scope_property_user_match)) {
+                config->auth_ldap->scope_property_user_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_ENDSWITH;
+              } else {
+                config_destroy(&cfg);
+                fprintf(stderr, "Error scope_property_user_match, values available are 'equals', 'contains', 'startswith' or 'endswith'\n");
+                return 0;
+              }
+            }
           }
           config->auth_ldap->name_property_user_read = o_strdup(cur_auth_ldap_name_property_user_read);
           if (config->auth_ldap->name_property_user_read == NULL) {
@@ -1282,6 +1340,21 @@ int build_config_from_file(struct config_elements * config) {
               fprintf(stderr, "Error allocating resources for config->auth_ldap->scope_property_client_read\n");
               return 0;
             }
+            if (cur_auth_ldap_scope_property_client_match != NULL) {
+              if (0 == o_strcasecmp("equals", cur_auth_ldap_scope_property_client_match)) {
+                config->auth_ldap->scope_property_client_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_EQUALS;
+              } else if (0 == o_strcasecmp("contains", cur_auth_ldap_scope_property_client_match)) {
+                config->auth_ldap->scope_property_client_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_CONTAINS;
+              } else if (0 == o_strcasecmp("startswith", cur_auth_ldap_scope_property_client_match)) {
+                config->auth_ldap->scope_property_client_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_STARTSWITH;
+              } else if (0 == o_strcasecmp("endswith", cur_auth_ldap_scope_property_client_match)) {
+                config->auth_ldap->scope_property_client_match = GLEWLWYD_SCOPE_PROPERTY_MATCH_ENDSWITH;
+              } else {
+                config_destroy(&cfg);
+                fprintf(stderr, "Error scope_property_client_match, values available are 'equals', 'contains', 'startswith' or 'endswith'\n");
+                return 0;
+              }
+            }
           }
           config->auth_ldap->name_property_client_read = o_strdup(cur_auth_ldap_name_property_client_read);
           if (config->auth_ldap->name_property_client_read == NULL) {
@@ -1497,6 +1570,25 @@ int build_config_from_file(struct config_elements * config) {
       fprintf(stderr, "Error, no jwt algorithm selected\n");
       return 0;
     }
+    // Perform autocheck of the jwt configuration to validate the certificates
+    time(&now);
+    session_token = generate_session_token(config, GLEWLWYD_CHECK_JWT_USERNAME, "127.0.0.1", now);
+    if (session_token != NULL) {
+      if (jwt_decode(&jwt_check, session_token, (const unsigned char *)config->jwt_decode_key, strlen(config->jwt_decode_key)) || jwt_get_alg(jwt_check) != jwt_get_alg(config->jwt)) {
+        fprintf(stderr, "Error autocheck jwt, step session_token validate\n");
+        o_free(session_token);
+        jwt_free(jwt_check);
+        config_destroy(&cfg);
+        return 0;
+      }
+      // JWT config valid
+      o_free(session_token);
+      jwt_free(jwt_check);
+    } else {
+      fprintf(stderr, "Error autocheck jwt, step session_token generate\n");
+      config_destroy(&cfg);
+      return 0;
+    }
   } else {
     config_destroy(&cfg);
     fprintf(stderr, "Error, no jwt parameters\n");
