@@ -2,7 +2,7 @@
  *
  * Glewlwyd OAuth2 Authorization Server
  *
- * OAuth2 authentiation server
+ * OAuth2 authentication server
  * Users are authenticated with a LDAP server
  * or users stored in the database 
  * Provides Json Web Tokens (jwt)
@@ -29,7 +29,7 @@
 #ifndef __GLEWLWYD_H_
 #define __GLEWLWYD_H_
 
-#define _GLEWLWYD_VERSION_ "1.2.3"
+#define _GLEWLWYD_VERSION_ "1.4.6"
 
 #include <jansson.h>
 #include <jwt.h>
@@ -44,6 +44,7 @@
 
 #include <crypt.h>
 #include <stdio.h>
+#include <ldap.h>
 
 /** Angharad libraries **/
 #define U_DISABLE_WEBSOCKET
@@ -58,8 +59,8 @@
   #error Libmicrohttpd version 0.9.38 minimum is required, you can download it at http://ftp.gnu.org/gnu/libmicrohttpd/
 #endif
 
-#define _GLEWLWYD_VERSION 1.1
 #define GLEWLWYD_LOG_NAME "Glewlwyd"
+#define GLEWLWYD_CHECK_JWT_USERNAME "myrddin"
 
 // Configuration default values
 #define GLEWLWYD_DEFAULT_PREFIX             "glewlwyd"
@@ -75,6 +76,7 @@
 #define GLEWLWYD_DEFAULT_LIMIT              20
 #define GLEWLWYD_PREFIX_BEARER              "Bearer "
 #define GLEWLWYD_DEFAULT_HASH_ALGORITHM     "MD5"
+#define GLEWLWYD_DEFAULT_LDAP_PAGE_SIZE     50
 
 #define GLEWLWYD_RESET_PASSWORD_DEFAULT_SMTP_PORT        25
 #define GLEWLWYD_RESET_PASSWORD_DEFAULT_TOKEN_EXPIRATION 604800
@@ -115,7 +117,7 @@
 
 // Authorization types available
 #define GLEWLWYD_AUHORIZATION_TYPE_AUTHORIZATION_CODE                  0
-#define GLEWLWYD_AUHORIZATION_TYPE_CODE                                1
+#define GLEWLWYD_AUHORIZATION_TYPE_AUTHORIZATION_CODE_ACCESS_TOKEN     1
 #define GLEWLWYD_AUHORIZATION_TYPE_IMPLICIT                            2
 #define GLEWLWYD_AUHORIZATION_TYPE_RESOURCE_OWNER_PASSWORD_CREDENTIALS 3
 #define GLEWLWYD_AUHORIZATION_TYPE_CLIENT_CREDENTIALS                  4
@@ -126,6 +128,12 @@
 #define GLEWLWYD_CALLBACK_PRIORITY_AUTHENTICATION 1
 #define GLEWLWYD_CALLBACK_PRIORITY_APPLICATION    2
 
+// LDAP Scope property match
+#define GLEWLWYD_SCOPE_PROPERTY_MATCH_EQUALS     0
+#define GLEWLWYD_SCOPE_PROPERTY_MATCH_CONTAINS   1
+#define GLEWLWYD_SCOPE_PROPERTY_MATCH_STARTSWITH 2
+#define GLEWLWYD_SCOPE_PROPERTY_MATCH_ENDSWITH   3
+
 pthread_mutex_t global_handler_close_lock;
 pthread_cond_t  global_handler_close_cond;
 
@@ -148,11 +156,14 @@ struct _auth_ldap {
   char *            bind_dn;
   char *            bind_passwd;
   struct crypt_data cur_crypt_data;
+  int               search_scope;
+	ber_int_t         page_size;
   
   char *  base_search_user;
   char *  filter_user_read;
   char *  login_property_user_read;
   char *  scope_property_user_read;
+  int     scope_property_user_match;
   char *  name_property_user_read;
   char *  email_property_user_read;
   char *  additional_property_value_read;
@@ -171,6 +182,7 @@ struct _auth_ldap {
   char *  filter_client_read;
   char *  client_id_property_client_read;
   char *  scope_property_client_read;
+  int     scope_property_client_match;
   char *  name_property_client_read;
   char *  description_property_client_read;
   char *  redirect_uri_property_client_read;
@@ -231,6 +243,7 @@ struct config_elements {
   unsigned int                    refresh_token_expiration;
   unsigned int                    access_token_expiration;
   unsigned int                    code_expiration;
+  unsigned int                    auth_code_match_ip_address;
   char *                          admin_scope;
   char *                          profile_scope;
   char *                          hash_algorithm;
@@ -337,7 +350,7 @@ json_t * get_user_list(struct config_elements * config, const char * source, con
 json_t * get_user_list_ldap(struct config_elements * config, const char * search, long int offset, long int limit);
 json_t * get_user_list_database(struct config_elements * config, const char * search, long int offset, long int limit);
 json_t * get_user(struct config_elements * config, const char * username, const char * source);
-json_t * get_user_profile(struct config_elements * config, const char * username, const char * source);
+json_t * get_user_profile(struct config_elements * config, const char * username, const char * source, const char * scope_list);
 json_t * get_user_database(struct config_elements * config, const char * username);
 json_t * get_user_ldap(struct config_elements * config, const char * username);
 json_t * get_user_http(struct config_elements * config, const char * username);
@@ -387,7 +400,7 @@ int add_resource(struct config_elements * config, json_t * j_resource);
 int set_resource(struct config_elements * config, const char * resource, json_t * j_resource);
 int delete_resource(struct config_elements * config, const char * resource);
 
-// Refesh token CRUD
+// Refresh token CRUD
 json_t * get_refresh_token_list(struct config_elements * config, const char * username, int valid, long int offset, long int limit);
 int revoke_token(struct config_elements * config, const char * username, const char * token_hash);
 
